/****************************************************************************
   Copyright (c) 1995, 1996, 1997  Intel Corporation.  All Rights Reserved.
****************************************************************************/

#ifndef INCLUDED_DINO_H
#define INCLUDED_DINO_H

#ifdef _WIN32
#define COM_NO_WINDOWS_H
#include <objbase.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

//
// Win32 DLL conventions for import/export of symbols
//

#define DllExport   __declspec( dllexport )
#define DllImport   __declspec( dllimport )

#ifdef _DINO_
#define DINO_SYMBOL DllExport
#else
#define DINO_SYMBOL DllImport
#endif

#define DINOFN      DINO_SYMBOL WINAPI

/****************************************************************************
	 Error and status code defines
****************************************************************************/

/* command completed successfully */
#define ERR_OK                          (0)

/* following error code denotes failure and reason when available */
#define ERR_FAILED                      (- 1)
#define ERR_OUT_OF_MEMORY               (- 2)
#define ERR_BAD_POINTER                 (- 3)
#define ERR_OUT_OF_RANGE                (- 4)
#define ERR_INVALID_SIZE                (- 5)                          
#define ERR_BAD_TYPE                    (- 6)
#define ERR_INVALID_ATTRIB              (- 7)
#define ERR_INVALID_DRAWORDER           (- 8)
#define ERR_INVALID_EFFECT              (- 9)
#define ERR_BAD_HANDLE_TYPE             (-10)
#define ERR_UNKNOWN_TSTYLE              (-11)
#define ERR_FILE_NOT_FOUND              (-12)
#define ERR_NOT_VALID_BMP               (-13)
#define ERR_FAILED_READING_FILE         (-14)
#define ERR_DIFFERENT_BMPS              (-15)
#define ERR_OBJECT_ALREADY_A_MEMBER     (-16)
#define ERR_OBJECT_NOT_A_MEMBER         (-17)
#define ERR_DATA_UNAVAILABLE            (-18)
#define ERR_NO_MSG_FN                   (-19) 
#define ERR_UNSUPPORTED                 (-20)
#define ERR_BADWIDTH                    (-21)
#define ERR_BADHEIGHT                   (-22)
#define ERR_UNSUPPORTED_COLORTYPE       (-23)
#define ERR_UNSUPPORTED_BITCOUNT        (-24)
#define ERR_BADDESTINATION              (-25)
#define ERR_BADHWND                     (-26)
#define ERR_BADEVENTTYPE                (-27)
#define ERR_BADFLAGS                    (-28) 
#define ERR_UNSUPPORTED_TSTYLE          (-29)
#define ERR_INVALID_RECTL               (-30) 
#define ERR_INVALID_USAGE               (-31)
#define ERR_BAD_FILENAME                (-32)                         
#define ERR_DCIMGR_UNAVAILABLE          (-33)
#define ERR_INVALIDCOLOR                (-34)
#define ERR_NOPRIORSETTING              (-35)
#define ERR_CANNOT_ACCESS_DISPLAY		(-36)
#define ERR_BAD_HANDLE                  (-37)
#define ERR_BAD_TILEWIDTH               (-38)
#define ERR_BAD_TILEHEIGHT              (-39)

// additional error messages just for timers
#define ERR_BADFREQ                     (-40)
#define ERR_TIMER_ALREADY_STARTED       (-41)
#define ERR_TIMER_FAILED                (-42)
#define ERR_TIMER_NOT_RUNNING           (-43)
#define ERR_CANNOT_KILL_TIMER           (-44)
#define ERR_INVALID_HOBJ                (-45)
#define ERR_INVALID_CONSTRAINT          (-46)
#define ERR_INVALID_CONDITION           (-47)
#define ERR_TIME_ALREADY_PAST           (-48)
#define ERR_INVALID_PERIOD              (-49)
#define ERR_EVENT_NOT_SCHEDULED         (-50)
                        
// joystick messages                        
#define ERR_TIMEOUT                     (-51)
#define ERR_NOP                         (-52)
// currently pentium is required since joystick code uses RDTSC
#define ERR_PENTIUM_REQUIRED            (-53)

// effect error messages
#define ERR_EFFECT_NOT_FOUND            (-60)
#define ERR_MEDIA_UNSUPPORTED           (-61)
#define ERR_INVALID_PARAMS              (-62)
#define ERR_BAD_PARAM_SIZE              (-63)
#define ERR_NOT_ENOUGH_FORMATS          (-64)

//collision error messages
#define ERR_BAD_COLL_DATA               (-65)
#define ERR_UNKNOWN_COLL_TYPE           (-66)

// Direct Draw error messages
#define ERR_CANNOT_ACCESS_DDRAW_SURFACE (-67)

#define ERR_TRANSPARENCY_NOT_SET        (-68)
#define ERR_VIDEO_MODE_CONFLICT         (-69)

#define ERR_EFFECT_UNSUPPORTED			(-70)	// custom effect error return code

#define ERR_INCOMPATIBLE_COLORTYPES		(-71)

#define ERR_REQUIRE_BITMAP              (-72)
#define ERR_REQUIRE_DESTINATION         (-73)

#define ERR_INTERNAL_SYS_ERROR          (-100)

/* status code denotes an invalid parameter was used and was adjusted appropriately */
#define ERR_CLIPPED_RANGE               (1)

//collision status returns
#define ERR_NO_COLLISION                (2)
#define ERR_NO_COLLISION_DATA           (3)

// AV error messages		
#define ERR_NO_BUFFERS                  (ERR_OK - 200)
#define ERR_AV_FILE_ERROR               (ERR_OK - 201)
#define ERR_NOT_PLAYING                 (ERR_OK - 202)
#define ERR_NOT_PAUSED                  (ERR_OK - 203)
#define ERR_CODEC_ERROR                 (ERR_OK - 204)
#define ERR_INVALID_POS                 (ERR_OK - 205)
#define ERR_AUDIO_ERROR                 (ERR_OK - 206)
#define ERR_AVNOCODEC                   (ERR_OK - 207)
#define ERR_AV_INUSE                    (ERR_OK - 208)	
#define ERR_NO_CMDS                     (ERR_OK - 209)	
#define ERR_INVALID_STATUSTYPE          (ERR_OK - 210)
#define ERR_INVALID_INFOTYPE            (ERR_OK - 211)
#define ERR_PLAY_PAST_END               (ERR_OK - 212)
#define ERR_CORRUPTED_TRACK             (ERR_OK - 213)
#define ERR_INVALID_TRACK               (ERR_OK - 214)
#define ERR_NO_MASTER_TRACK             (ERR_OK - 215)
#define ERR_NO_STREAMS                  (ERR_OK - 216)
#define ERR_INVALID_STREAM              (ERR_OK - 217)
#define ERR_CORRUPTED_FIL               (ERR_OK - 218)
#define ERR_UNSUPPORTED_AUDIO_FORMAT    (ERR_OK - 219)
#define ERR_ALREADY_PLAYING             (ERR_OK - 220)
#define ERR_OUT_OF_RESOURCES            (ERR_OK - 221)
#define ERR_DS_TURNED_OFF               (ERR_OK - 222)
#define ERR_DS_NO_DLL                   (ERR_OK - 223)
#define ERR_DS_INVALID_DLL              (ERR_OK - 224)
#define ERR_DS_NO_DRIVER                (ERR_OK - 225)
#define ERR_DS_ALREADY_IN_USE           (ERR_OK - 226)
#define ERR_DS_BAD_FORMAT               (ERR_OK - 227)
#define ERR_AUDIO_NOT_AVAILABLE         (ERR_OK - 228)
#define ERR_TOO_MANY_ACTIVE_TRACKS      (ERR_OK - 229)
#define ERR_INVALID_HWND                (ERR_OK - 230)
#define ERR_INVALID_DESTROYTYPE         (ERR_OK - 231)
#define ERR_ALREADY_DISCONNECTED        (ERR_OK - 232)
#define ERR_NOT_DISCONNECTED            (ERR_OK - 233)
#define ERR_AUDIO_DISCONNECTED          (ERR_OK - 234)

//RDXAM error returns 
#define ERR_AM_NOT_FOUND				(ERR_OK - 235)
#define ERR_AM_RENDER_FAILED			(ERR_OK - 236)

// Renamed error codes from RDX 2.0 to RDX 2.1
//	These are here for backwards compatibility.

#define ERR_CANNOT_ACCESS_DCI_SURFACE   ERR_CANNOT_ACCESS_DISPLAY
#define ERR_NOT_PCM                     ERR_UNSUPPORTED_AUDIO_FORMAT

/************************************************************************************
	Object management functions and structures
************************************************************************************/

typedef LONG DINORVAL;

/* bitmaps */

typedef void	*HBMP;
typedef HBMP	HSEQ;

typedef struct 
{
	DWORD dwBufSize;	// size of hBmp buffer in bytes
	DWORD dwBufStride;	// stride of hBmp buffer in bytes
} HBMPHEADER;

typedef enum {D2D_BACKGROUND = 1, D2D_SPRITE, D2D_TILE} USAGE;

/* backgrounds */
typedef void * HBKG;

/* sprites */
typedef void * HSPR;

/*  tiles */
typedef void * HTILE;

/* grids */
typedef struct
{
	WORD mTile;
	WORD nTile;
} LOCATION;

typedef void * HGRID;

// color type is used in CreateBitmap & CreateSurface
typedef enum {RGB_CLUT8, RGB_555, RGB_565, RGB_24, RGB_32, RGB_ALPHA } COLORTYPE;

/* for bitmaps */

#define RGB555(r,g,b)	( ((WORD)((b) & 0xf8) >> 3)	\
						| ((WORD)((g) & 0xf8) << 2)	\
						| ((WORD)((r) & 0xf8) << 7))

#define RGB565(r,g,b)	( ((WORD)((b) & 0xf8) >> 3)	\
						| ((WORD)((g) & 0xfc) << 3)	\
						| ((WORD)((r) & 0xf8) << 8))

DINORVAL DINOFN hbmpCreate(DWORD dwWidth, DWORD dwHeight,
								COLORTYPE ColorType, HBMP *phBmp);
DINORVAL DINOFN hbmpDelete(HBMP hBmp);
DINORVAL DINOFN hbmpLoad(HBMP hBmp, POINT pt, PSTR pFilename, RECT rSrc);
DINORVAL DINOFN hbmpSetPointer(HBMP hBmp, BYTE *pBits, DWORD dwStride);
DINORVAL DINOFN hbmpGetBits(BYTE * pBits, POINT DstPt, DWORD dwStride,
                                HBMP hBmp, RECT rSrc);
DINORVAL DINOFN hbmpGetLockedBuffer(HBMP hBmp, BYTE **pphBmpBuf, HBMPHEADER *phBmpHeader);
DINORVAL DINOFN hbmpReleaseBuffer(HBMP hBmp);
DINORVAL DINOFN hbmpGetSize(HBMP hBmp, DWORD * pWidth, DWORD *pHeight);
DINORVAL DINOFN hbmpPrepare(HBMP hBmp, USAGE usage, BOOL fFlip );
DINORVAL DINOFN hbmpSetTransparencyColor (HBMP hBmp, DWORD color);
DINORVAL DINOFN hbmpGetTransparencyColor (HBMP hBmp, DWORD * pColor);
DINORVAL DINOFN hbmpSetTransparencyMask (HBMP hBmp, BYTE * pMask);
DINORVAL DINOFN hbmpGetTransparencyMask (HBMP hBmp, BYTE * pMask);
DINORVAL DINOFN hbmpClearTransparency (HBMP hBmp);
DINORVAL DINOFN hbmpSetCompressedImage(HBMP hBmp, BYTE *pCodes, DWORD dwLength);
DINORVAL DINOFN hbmpGetCompressedImage(HBMP hBmp, BYTE *pCodes, DWORD *pLength);

/* for Sequences */

DINORVAL DINOFN hseqCreate(UINT count, UINT width, UINT height, 
	COLORTYPE color, HSEQ *phSeq);
DINORVAL DINOFN hseqSetData(HSEQ hSeq, UINT index, HBMP hBmp);
DINORVAL DINOFN hseqGetData(HSEQ hSeq, UINT index, HBMP *phBmp);
DINORVAL DINOFN hseqDelete(HSEQ hSeq);

/* for Sprites */
typedef enum
{
    SPR_FLAG_HWBLIT = 0x01, // Uses hardware based image blits from video RAM.
    SPR_FLAG_ALL    = 0x01,

} SPR_FLAGS;

typedef struct
{
    GUID    id;         // identifies the file format
    DWORD   dwWidth;    // size of the decoded image
    DWORD   dwHeight;
    DWORD   dwBitDepth; // 8 or 24 bits per pixel
    DWORD   dwCKey;     // pixel color originally used for transparency or ~0
    DWORD   dwSize;     // number of bytes in the encoded image

} SPR_HEADER;

#ifdef _DINO_
DINO_SYMBOL DEFINE_GUID(SPRITE_GUID,0x9c8a5a80,0x3718,0x11d0,0xbf,0x20,0x0,0xa0,0xc9,0x1a,0x26,0x29);
#else
DINO_SYMBOL GUID SPRITE_GUID;
#endif

DINORVAL DINOFN sprCreate(HSPR *phSpr);
DINORVAL DINOFN sprSetData(HSPR hSpr, HBMP hBmp);
DINORVAL DINOFN sprGetData(HSPR hSpr, HBMP *phBmp);
DINORVAL DINOFN sprDelete(HSPR hSpr);
DINORVAL DINOFN sprSetFlags(HSPR hSpr, SPR_FLAGS flags);
DINORVAL DINOFN sprGetFlags(HSPR hSpr, SPR_FLAGS *pflags);
DINORVAL DINOFN sprClearFlags(HSPR hSpr, SPR_FLAGS flags);

/* for Backgrounds */
typedef enum
{
	BKG_FLAG_HWBLIT     = 0x01, // Uses hardware based image blits from video RAM.
	BKG_FLAG_ALL		= 0x01,

} BKG_FLAGS;

DINORVAL DINOFN bkgCreate(HBKG *phBkg);
DINORVAL DINOFN bkgSetData(HBKG hBkg, HBMP hBmp);
DINORVAL DINOFN bkgGetData(HBKG hBkg, HBMP *phBmp);
DINORVAL DINOFN bkgDelete(HBKG hBkg);
DINORVAL DINOFN bkgSetFlags(HBKG hBkg, BKG_FLAGS flags);
DINORVAL DINOFN bkgGetFlags(HBKG hBkg, BKG_FLAGS *pflags);
DINORVAL DINOFN bkgClearFlags(HBKG hBkg, BKG_FLAGS flags);

/* for Tiles */
DINORVAL DINOFN tleCreate(HTILE *phTile);
DINORVAL DINOFN tleSetData(HTILE hTile, HBMP hBmp);
DINORVAL DINOFN tleGetData(HTILE hTile, HBMP *phBmp);
DINORVAL DINOFN tleDelete(HTILE hTile);

/* for Grids */

DINORVAL DINOFN grdCreateEx(WORD wTilesWide, WORD wTilesHigh, 
	WORD wTileWidth, WORD wTileHeight, COLORTYPE iColorType, HGRID *phGrid);

DINORVAL DINOFN grdSetData(HGRID hGrid, LOCATION loc, HTILE hTile);
DINORVAL DINOFN grdGetData(HGRID hGrid, LOCATION loc, HTILE *phTile);
DINORVAL DINOFN grdClearData(HGRID hGrid, LOCATION loc);
DINORVAL DINOFN grdDelete(HGRID hGrid);

// grdCreate() is for backwards compatibility with RDX 2.0 and is now obsolete.
DINORVAL DINOFN grdCreate(WORD wTilesWide, WORD wTilesHigh, 
	WORD wTileWidth, WORD wTileHeight, HGRID *phGrid);

/************************************************************************************
	Object attribute manipulation functions and structures, surface management functions
************************************************************************************/

typedef void * HOBJ;
typedef void * HSURF;
typedef void * HGRP;
typedef void * HTIMER; 
typedef void * HFX;
typedef unsigned long HEVENT; // Matches with AM definition

//COLLISION
#define COLLISION_RECT_TYPE 1

typedef struct
{
	RECT	region;
    DWORD	dwRegionID;
} COLLISION_REGION;


typedef struct
{
	DWORD 	dwCDType;
    DWORD	dwSizeCDBuf;
    VOID	*pCDBuf;
}COLLISION_DATA_CONTEXT;


typedef struct
{
	DWORD	nRegions;
    COLLISION_REGION *pCRArray;
} COLLISION_RECT_DATA;


typedef struct
{
	HOBJ hobj1;
    DWORD dwRegionID1;
    HOBJ hobj2;
    DWORD dwRegionID2;
} COLLISION_PAIR;


typedef struct tagRECTWH
{
	long x;
	long y;
	long dx;	// should be positive always
	long dy;	// should be positive always
} RECTWH;

//
// Effect GUIDs
//
// FX_HORIZ_FLIP      78B22763-0F77-11cf-95D6-00AA006D7CE1
// FX_VERT_FLIP       78B22764-0F77-11cf-95D6-00AA006D7CE1
// FX_HORIZ_SHEAR     78B22761-0F77-11cf-95D6-00AA006D7CE1
// FX_VERT_SHEAR      78B22762-0F77-11cf-95D6-00AA006D7CE1
// FX_SMORPH          78B22760-0F77-11cf-95D6-00AA006D7CE1
//
// Effect GUIDs (supported only by surfaces)
//
// FX_RENDER_CALLBACK 04D591C1-1DC2-11cf-95D6-00AA006D7CE1
// FX_MODEX_DISPLAY   04D591C2-1DC2-11cf-95D6-00AA006D7CE1
// FX_FULL_SCREEN     04D591C0-1DC2-11cf-95D6-00AA006D7CE1
// FX_MAP_CLUT8       FE576D81-6C5B-11cf-95D6-00AA006D7CE1
// FX_MAP_RGB16BIT    F601BD60-FF41-11cf-9B66-00AA006D7CE1
// FX_DDRAW_RENDER_CALLBACK DDF26161-A3F8-11cf-B8C8-00AA00B7E179

#ifdef _DINO_
DINO_SYMBOL DEFINE_GUID(FX_HORIZ_FLIP,0x78B22763,0xF77,0x11CF,0x95,0xD6,0x0,0xAA,0x0,0x6D,0x7C,0xE1);
DINO_SYMBOL DEFINE_GUID(FX_VERT_FLIP,0x78B22764,0xF77,0x11CF,0x95,0xD6,0x0,0xAA,0x0,0x6D,0x7C,0xE1);
DINO_SYMBOL DEFINE_GUID(FX_HORIZ_SHEAR,0x78B22761,0xF77,0x11CF,0x95,0xD6,0x0,0xAA,0x0,0x6D,0x7C,0xE1);
DINO_SYMBOL DEFINE_GUID(FX_VERT_SHEAR,0x78B22762,0xF77,0x11CF,0x95,0xD6,0x0,0xAA,0x0,0x6D,0x7C,0xE1);
DINO_SYMBOL DEFINE_GUID(FX_SMORPH,0x78B22760,0xF77,0x11CF,0x95,0xD6,0x0,0xAA,0x0,0x6D,0x7C,0xE1);
DINO_SYMBOL DEFINE_GUID(FX_RENDER_CALLBACK,0x4D591C1,0x1DC2,0x11CF,0x95,0xD6,0x0,0xAA,0x0,0x6D,0x7C,0xE1);
DINO_SYMBOL DEFINE_GUID(FX_MODEX_DISPLAY,0x4D591C2,0x1DC2,0x11CF,0x95,0xD6,0x0,0xAA,0x0,0x6D,0x7C,0xE1);
DINO_SYMBOL DEFINE_GUID(FX_FULL_SCREEN,0x4D591C0,0x1DC2,0x11CF,0x95,0xD6,0x0,0xAA,0x0,0x6D,0x7C,0xE1);
DINO_SYMBOL DEFINE_GUID(FX_MAP_CLUT8,0xFE576D81,0x6C5B,0x11CF,0x95,0xD6,0x0,0xAA,0x0,0x6d,0x7C,0xE1);
DINO_SYMBOL DEFINE_GUID(FX_MAP_RGB16BIT,0xf601bd60,0xff41,0x11cf,0x9b,0x66,0x0,0xaa,0x0,0x6d, 0x7c, 0xe1);
DINO_SYMBOL DEFINE_GUID(FX_DDRAW_RENDER_CALLBACK,0xddf26161, 0xa3f8, 0x11cf, 0xb8, 0xc8, 0x0, 0xaa, 0x0, 0xb7, 0xe1, 0x79);
#else
DINO_SYMBOL GUID FX_HORIZ_FLIP;
DINO_SYMBOL GUID FX_VERT_FLIP;
DINO_SYMBOL GUID FX_HORIZ_SHEAR;
DINO_SYMBOL GUID FX_VERT_SHEAR;
DINO_SYMBOL GUID FX_SMORPH;
DINO_SYMBOL GUID FX_RENDER_CALLBACK;
DINO_SYMBOL GUID FX_MODEX_DISPLAY;
DINO_SYMBOL GUID FX_FULL_SCREEN;
DINO_SYMBOL GUID FX_MAP_CLUT8;
DINO_SYMBOL GUID FX_MAP_RGB16BIT;
DINO_SYMBOL GUID FX_DDRAW_RENDER_CALLBACK;
#endif

typedef signed long FixedPoint_S_15_16;   // signed 15 bit int, 16 bit fraction
typedef struct tagSMORPH_VALUES
{
	FixedPoint_S_15_16 StartX; 
	FixedPoint_S_15_16 StartY; 
	FixedPoint_S_15_16 DeltaX; 
	FixedPoint_S_15_16 DeltaY; 
}SMORPH_VALUES;

typedef struct tagSMORPH_PARAMS
{   
	long wrap;		// 0 if clipping enabled, else wrap
	SMORPH_VALUES * pSmorphArray;	// variable sized, atleast 1
} SMORPH_PARAMS;   

//
// RENDER_CALLBACK
//

#define RENDER_CALLBACK	0x00000001
                    
typedef struct tagRENDER_CALLBACK_PARAMS
{
	DINORVAL (WINAPI *pRenderCallback)(HSURF, BYTE *pBits, long lStride, void *pParams);
	void * pContext;
} RENDER_CALLBACK_PARAMS;



#ifdef __DDRAW_INCLUDED__
typedef struct tagDDRAW_RENDER_CALLBACK_PARAMS
{
	DINORVAL (WINAPI *pRenderCallback) (HSURF,
										LPDIRECTDRAW,
										LPDIRECTDRAWSURFACE,
										void * pContext);
	void *pContext;
} DDRAW_RENDER_CALLBACK_PARAMS;
#endif


typedef struct tagFULL_SCREEN_PARAMS
{
	DWORD		dwWidth, 
				dwHeight;
	COLORTYPE	iColorType;

} FULL_SCREEN_PARAMS;

typedef struct tagCOLORMAP_ENTRY
{
	BYTE	nRed, 	// Target RGB color value.
			nGreen,
			nBlue, 
			nIndex;	// Target CLUT8 color index

} COLORMAP_ENTRY, COLORMAP_TABLE[256];

typedef enum
{
	SURF_FLAG_HWBLIT		= 0x01,
	SURF_FLAG_PAGEFLIP		= 0x02,
	SURF_FLAG_HWCOLORFILL	= 0x04,
	SURF_FLAG_WAIT4VBI		= 0x08,
	SURF_FLAG_CLIPPING		= 0x10,

	SURF_FLAG_ALL			= 0x1F,
} SURF_FLAGS;

/* for surfaces */
DINORVAL DINOFN srfCreate(DWORD dwWidth, DWORD dwHeight, 
								COLORTYPE ColorType, HSURF *phSurf);                                                
DINORVAL DINOFN srfDelete(HSURF hSurf);
DINORVAL DINOFN srfSetColor(HSURF hSurf, DWORD dwColor);
DINORVAL DINOFN srfGetColor(HSURF hSurf, DWORD * pdwColor);
DINORVAL DINOFN srfClearColor(HSURF hSurf);
DINORVAL DINOFN srfSetFlags(HSURF hSurf, SURF_FLAGS flags);
DINORVAL DINOFN srfGetFlags(HSURF hSurf, SURF_FLAGS *pFlags);
DINORVAL DINOFN srfClearFlags(HSURF hSurf, SURF_FLAGS flags);
DINORVAL DINOFN srfSetDestWindow(HSURF hSurf, HWND hWnd);
DINORVAL DINOFN srfGetDestWindow(HSURF hSurf, HWND *phWnd);
DINORVAL DINOFN srfSetDestMemory(HSURF hSurf, BYTE *pBuffer, long lStride);
DINORVAL DINOFN srfGetDestMemory(HSURF hSurf, BYTE **pBuffer, long *plStride);
DINORVAL DINOFN srfDraw(HSURF hSurf);


/* attribute manipulation functions */
DINORVAL DINOFN objSetDrawOrder(HOBJ hObj, DWORD dwDrawOrder);
DINORVAL DINOFN objAdjDrawOrder(HOBJ hObj, long lDrawOrder);
DINORVAL DINOFN objGetDrawOrder(HOBJ hObj, DWORD *pdwDrawOrder);
DINORVAL DINOFN objSetVisibility(HOBJ hObj, long lDisplay);
DINORVAL DINOFN objAdjVisibility(HOBJ hObj, long lDisplay);
DINORVAL DINOFN objGetVisibility(HOBJ hObj, long *plDisplay);
DINORVAL DINOFN objSetPosition(HOBJ hObj, POINT pt);
DINORVAL DINOFN objAdjPosition(HOBJ hObj, POINT pt);
DINORVAL DINOFN objGetPosition(HOBJ hObj, POINT *pPt);
DINORVAL DINOFN objSetView(HOBJ hObj, RECTWH rwhSrc);
DINORVAL DINOFN objAdjView(HOBJ hObj, RECTWH rwhSrc);
DINORVAL DINOFN objGetView(HOBJ hObj, RECTWH *prwhSrc);
DINORVAL DINOFN objSetCurrentImage(HOBJ hObj, long lIndex);
DINORVAL DINOFN objAdjCurrentImage(HOBJ hObj, long lIndex);
DINORVAL DINOFN objGetCurrentImage(HOBJ hObj, long *plIndex);
DINORVAL DINOFN objSetDestRect(HOBJ hObj, RECTWH rwhDest);
DINORVAL DINOFN objAdjDestRect(HOBJ hObj, RECTWH rwhDest);
DINORVAL DINOFN objGetDestRect(HOBJ hObj, RECTWH *prwhDest);
DINORVAL DINOFN objSetDestination(HOBJ hObj, HSURF hSurf);
DINORVAL DINOFN objGetDestination(HOBJ hObj, HSURF *phSurf);
DINORVAL DINOFN objApplyEffect(HOBJ hObj, GUID fxID, void *pParams, HFX *phFx);
DINORVAL DINOFN objRemoveEffect(HOBJ hObj, HFX hFx);
DINORVAL DINOFN objPrepare(HOBJ hObj);

/* collision functions */
DINORVAL DINOFN objSetCollisionData (HOBJ hobj, COLLISION_DATA_CONTEXT  *pCDC);
DINORVAL DINOFN objGetCollisionData (HOBJ hobj, 
									 DWORD dwSizeOfBuf, 
									 VOID *pBuf, 
									 COLLISION_DATA_CONTEXT  *pCDC);

DINORVAL DINOFN objClearCollisionData (HOBJ hobj);

DINORVAL DINOFN objTestCollisionData (HOBJ hobj1, HOBJ hobj2,COLLISION_PAIR *pCollList);
DINORVAL DINOFN objTestCollisionPt (HOBJ hobj, POINT pt ,COLLISION_PAIR *pCollList);


/* effect functions */
DINORVAL DINOFN fxSetActive(HFX hFx, BOOL bState);
DINORVAL DINOFN fxGetActive(HFX hFx, BOOL *pbState);
DINORVAL DINOFN fxSetParams(HFX hFx, void *pParams);
DINORVAL DINOFN fxGetParams(HFX hFx, void *pParams);
DINORVAL DINOFN fxSetLimits(HFX hFx, DWORD dwInWidth, DWORD dwInHeight,
			DWORD dwOutWidth, DWORD dwOutHeight, void *pParams);
DINORVAL DINOFN fxGetLimits(HFX hFx, DWORD *pdwInWidth, DWORD *pdwInHeight,
			DWORD *pdwOutWidth, DWORD *pdwOutHeight, void *pParams, int size);


/* group functions */
DINORVAL DINOFN grpCreate (HGRP *phGrp);
DINORVAL DINOFN grpDelete (HGRP hGrp);
DINORVAL DINOFN grpAddObject (HGRP hGrp, HOBJ hObj);
DINORVAL DINOFN grpRemoveObject(HGRP hGrp, HOBJ hObj);
DINORVAL DINOFN grpListObjects (HGRP hGrp, HOBJ *lpBuffer, WORD wBufferSize);
DINORVAL DINOFN grpGetNumObjects (HGRP hGrp, long *pNum);

/* timer functions */
DINORVAL DINOFN timerCreate(WORD TimerFreq, HTIMER *phTimer);
DINORVAL DINOFN timerSetFrequency(HTIMER hTimer, WORD TimerFreq);
DINORVAL DINOFN timerGetFrequency(HTIMER hTimer, WORD *pTimerFreq);
DINORVAL DINOFN timerStart(HTIMER hTimer);
DINORVAL DINOFN timerReset(HTIMER hTimer);
DINORVAL DINOFN timerStop(HTIMER hTimer);
DINORVAL DINOFN timerDelete(HTIMER hTimer);
DINORVAL DINOFN timerGetCurrentTick(HTIMER hTimer, DWORD *pTick);
DINORVAL DINOFN timerSetCurrentTick(HTIMER hTimer, DWORD Tick);

/* event functions */
/* Used by eventCreate() */
typedef enum
{
	EVENT_SET_ATTRIB		= 1,
	EVENT_ADJUST_ATTRIB		= 2,
	EVENT_SET_EFFECT_ACTIVE	= 3,
	EVENT_DRAW				= 5,
	EVENT_CALLBACK			= 7,
	EVENT_USER				= 1000,

} EVENT_TYPE;

/* for events, used in eventCreate() */  
typedef enum
{
	ATTRIB_DRAWORDER	= 1,
	ATTRIB_VISIBILITY	= 2,
	ATTRIB_POSX			= 3,
	ATTRIB_POSY			= 4,
	ATTRIB_VIEWX		= 5,
	ATTRIB_VIEWDX		= 6,
	ATTRIB_VIEWY		= 7,
	ATTRIB_VIEWDY		= 8,
	ATTRIB_CURRENTIMAGE	= 9

} ATTRIB;

typedef enum
{
	NO_CONSTRAINT = 0,
	ADJUSTBOUNDS,
	RANDOMNESS,

} EVENT_CONSTRAINT;

typedef enum
{
	NO_STYLE = 0,
	BOUNCE,
	CYCLE,

} EVENT_STYLE;

typedef enum
{
	NO_CONDITION 			= 0x00,
	EVENT_COMPLETE			= 0x01,
	ATTRIB_ADJUST			= 0x02,
	ATTRIB_ADJUST_OVERFLOW	= 0x04,

} EVENT_CONDITION;

// for flags in eventSchedule
typedef enum
{
	RELATIVE_TIME = 1,
	ABSOLUTE_TIME = 2,

} TIMER_FLAGS;

typedef DINORVAL (WINAPI EventFn)(HEVENT hEvent, DWORD msg, DWORD type, DWORD dwContext);

DINORVAL DINOFN eventCreate(HOBJ hObj, EVENT_TYPE dwEventType, 
							long lParam1, long lParam2, HEVENT *phEvent);
DINORVAL DINOFN eventDelete (HEVENT hEvent);
DINORVAL DINOFN eventContinue(HEVENT hEvent, long lTimes);
DINORVAL DINOFN eventSchedule (HTIMER hTimer, HEVENT hEvent, DWORD dwStartTick, 
								TIMER_FLAGS flags, WORD wPeriod, DWORD dwTimes);
DINORVAL DINOFN eventSetAdjustConstraint (HEVENT hEvent, EVENT_CONSTRAINT iConstraint, 
									EVENT_STYLE iStyle, long lMin, long lMax);
DINORVAL DINOFN eventGetAdjustConstraint (HEVENT hEvent, EVENT_CONSTRAINT iConstraint, 
									EVENT_STYLE *piStyle, long *plMax, long *plMin);
DINORVAL DINOFN eventClearAdjustConstraint (HEVENT hEvent, EVENT_CONSTRAINT iConstraint);
DINORVAL DINOFN eventSetConstraintCallback (HEVENT hEvent, EVENT_CONDITION iCondition, 
										EventFn *pfnCB, long lContext);
DINORVAL DINOFN eventGetConstraintCallback (HEVENT hEvent, EVENT_CONDITION iCondition, 
										EventFn *pfnCB, long * plContext);

/* for devices */
#define	JOYSTICK0	1		// joystick num 0
#define	JOYSTICK1	2		// joystick num 1

DINORVAL DINOFN devStartJoystick(int StickNum);
DINORVAL DINOFN devReadJoystick(int iStick, DWORD *pdwXPos, DWORD *pdwYPos, BYTE *bButtons);

DINORVAL DINOFN devSetPalette (DWORD dwNumValues, DWORD dwStart, RGBQUAD * pValues);
DINORVAL DINOFN devGetPalette (DWORD dwNumValues, DWORD dwStart, RGBQUAD * pValues); 

DINORVAL DINOFN devQueryVideoMode(int iIndex, DWORD *pWidth, DWORD *pHeight, 
	COLORTYPE *pColorType, void *pReserved);

#ifdef __cplusplus
}
#endif // __cplusplus

#endif // INCLUDED_DINO_H


